@@ -1,13 +1,20 @@
-from zope.interface import implements
-from zope.component import adapts, queryAdapter
+from zope.interface import implementer
+from zope.component import adapter, queryAdapter
 
 from persistent.dict import PersistentDict
 
 from zope.annotation.interfaces import IAnnotations
+from zope.component import getUtility
 
 from AccessControl import getSecurityManager
-from Products.CMFCore.utils import getToolByName
-from webdav.LockItem import LockItem
+try:
+    from OFS.LockItem import LockItem
+except ImportError:
+    # Zope2
+    from webdav.LockItem import LockItem
+
+from plone.registry.interfaces import IRegistry
+from Products.CMFPlone.interfaces import IEditingSchema
 
 from plone.locking.interfaces import IRefreshableLockable
 from plone.locking.interfaces import INonStealableLock
@@ -15,44 +22,53 @@ from plone.locking.interfaces import ITTWLockable
 from plone.locking.interfaces import STEALABLE_LOCK
 from plone.locking.interfaces import ILockSettings
 
-ANNOTATION_KEY = 'plone.locking'
-DEFAULT_TIMEOUT = 10 * 60L
+try:
+    from plone.protect.auto import safeWrite
+except ImportError:
+    def safeWrite(*args):
+        pass
 
+
+ANNOTATION_KEY = 'plone.locking'
+
+
+@adapter(ITTWLockable)
+@implementer(IRefreshableLockable)
 class TTWLockable(object):
     """An object that is being locked through-the-web
     """
 
-    implements(IRefreshableLockable)
-    adapts(ITTWLockable)
-
     def __init__(self, context):
         self.context = context
         self.__locks = None
-        
+
     def lock(self, lock_type=STEALABLE_LOCK, children=False):
         settings = queryAdapter(self.context, ILockSettings)
         if settings is None:
-            # No context specific adapter, is this a Plone site?
-            pprops = getToolByName(self.context, 'portal_properties', None)
-            if pprops is not None and 'site_properties' in pprops.objectIds():
-                settings = pprops.site_properties
+            registry = getUtility(IRegistry)
+            settings = registry.forInterface(IEditingSchema,
+                                             prefix='plone')
         if settings is not None and settings.lock_on_ttw_edit is False:
             return
-        
+
         if not self.locked():
             user = getSecurityManager().getUser()
             depth = children and 'infinity' or 0
-            lock = LockItem(user, depth=depth, timeout=DEFAULT_TIMEOUT)
+            timeout = int(lock_type.timeout * 60)
+            lock = LockItem(user, depth=depth, timeout=timeout)
             token = lock.getLockToken()
+            self.context._v_safe_write = True
             self.context.wl_setLock(token, lock)
 
-            self._locks()[lock_type.__name__] = dict(type = lock_type,
-                                                  token = token)
-    
+            locks = self._locks()
+            locks[lock_type.__name__] = dict(type=lock_type, token=token)
+            safeWrite(locks)
+            safeWrite(self.context)
+
     def refresh_lock(self, lock_type=STEALABLE_LOCK):
         if not self.locked():
             return
-            
+
         key = self._locks().get(lock_type.__name__, None)
         if key:
             lock = self.context.wl_getLock(key['token'])
@@ -76,7 +92,7 @@ class TTWLockable(object):
     def locked(self):
         if self.lock_info():
             return True
-        else: 
+        else:
             return False
 
     def can_safely_unlock(self, lock_type=STEALABLE_LOCK):
@@ -108,7 +124,7 @@ class TTWLockable(object):
             if not hasattr(l['type'], '__name__') or \
                l['type'].__name__ != lock_type.__name__:
                 return False
-        # The lock type is stealable, and the object is not marked as 
+        # The lock type is stealable, and the object is not marked as
         # non-stelaable, so return True
         if not INonStealableLock.providedBy(self.context):
             return True
@@ -119,20 +135,26 @@ class TTWLockable(object):
     def lock_info(self):
         info = []
         rtokens = dict([(v['token'], v['type']) for v in self._locks(False).values()])
-        isReadOnly = self.context._p_jar.isReadOnly()
-        for lock in self.context.wl_lockValues(not isReadOnly):
+        jar = self.context._p_jar
+        if jar is not None:
+            isReadOnly = jar.isReadOnly()
+        else:
+            isReadOnly = False
+        lock_mapping = self.context.wl_lockmapping(not isReadOnly)
+        safeWrite(lock_mapping)
+        for lock in lock_mapping.values():
             if not lock.isValid():
-                continue # Skip invalid/expired locks
+                continue  # Skip invalid/expired locks
             token = lock.getLockToken()
             creator = lock.getCreator()
             # creator can be None when locked by an anonymous user
             if creator is not None:
                 creator = creator[1]
             info.append({
-                'creator' : creator,
-                'time'    : lock.getModifiedTime(),
-                'token'   : token,
-                'type'    : rtokens.get(token, None),
+                'creator': creator,
+                'time': lock.getModifiedTime(),
+                'token': token,
+                'type': rtokens.get(token, None),
             })
         return info
 
@@ -144,6 +166,12 @@ class TTWLockable(object):
         locks = annotations.get(ANNOTATION_KEY, None)
         if locks is None and create:
             locks = annotations.setdefault(ANNOTATION_KEY, PersistentDict())
+
+        try:
+            safeWrite(annotations.obj.__annotations__)
+        except AttributeError:
+            pass
+
         if locks is not None:
             self.__locks = locks
             return self.__locks
